#![feature(proc_macro_hygiene, decl_macro)]

#[macro_use]
extern crate rocket;
#[macro_use]
extern crate rocket_contrib;
#[macro_use]
extern crate diesel;
#[macro_use]
extern crate serde_derive;

pub mod models;
pub mod schema;
use crate::models::Klassen;
use crate::schema::klassen;
use diesel::RunQueryDsl;
use rocket::request::{Form, FromParam};
use rocket::response::Redirect;
use rocket_contrib::databases;
use rocket_contrib::serve::StaticFiles;
use rocket_contrib::templates::Template;

struct StundenByID(i32);
#[derive(Debug)]
enum WebError {
    I32NotParsed,
}

impl<'a> FromParam<'_> for StundenByID {
    type Error = WebError;
    fn from_param(param: &rocket::http::RawStr) -> Result<Self, Self::Error> {
        if let Ok(n) = param.parse::<i32>() {
            Ok(StundenByID(n))
        } else {
            Err(WebError::I32NotParsed)
        }
    }
}

#[database("page_db")]
struct DbConn(databases::diesel::SqliteConnection);

#[derive(Debug, Serialize)]
struct ListAllContext {
    klassen: Vec<Klassen>,
    parent: String,
}

#[get("/")]
fn index(conn: DbConn) -> Template {
    Template::render(
        "uebersicht",
        ListAllContext {
            klassen: klassen::dsl::klassen.load(&*conn).expect("database acess"),
            parent: "base".to_string(),
        },
    )
}

#[get("/add/klasse")]
fn get_klasse_form() -> Template {
    Template::render("add_klasse_form", "")
}

#[derive(FromForm, Insertable)]
#[table_name = "klassen"]
struct KlasseForm {
    stufe: i32,
    gruppe: String,
    titel: Option<String>,
    ordnung: i32,
}

#[post("/add/klasse", data = "<klasse>")]
fn post_klasse_form(klasse: Form<KlasseForm>, conn: DbConn) -> Result<Redirect, String> {
    match diesel::insert_into(schema::klassen::table)
        .values(klasse.into_inner())
        .execute(&*conn)
    {
        Ok(_) => Ok(Redirect::to("/admin/add/klasse")),
        Err(_) => Err("Klasse konnte nicht erstellt werden!".to_string()),
    }
}

fn main() {
    rocket::ignite()
        .attach(DbConn::fairing())
        .attach(Template::fairing())
        .mount(
            "/static",
            StaticFiles::from(concat!(env!("CARGO_MANIFEST_DIR"), "/static")),
        )
        .mount("/", routes![index])
        .mount("/admin/", routes![get_klasse_form])
        .mount("/admin/", routes![post_klasse_form])
        .launch();
}
